// $Id: padma.js,v 1.3.2.15 2005/07/18 17:27:21 vnagarjuna Exp $ -->

//Copyright 2005 Nagarjuna Venna <vnagarjuna@yahoo.com>

/* ***** BEGIN LICENSE BLOCK ***** 
 *  This file is part of Padma.
 *
 *  Padma is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Padma is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Padma; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * ***** END LICENSE BLOCK ***** */

//Broswer - both automatic and manual transformation start here

var Padma_Browser_Transformer = {

    //Common stuff

    inputMethod: Transformer.method_Unknown,
    outputMethod: Transformer.method_Unknown,
    rtsStartMode: RTSTransformer.rtsEnglish,
    transfomer: null,

    //Heuristic Transform
    heuristicTransformSites: ["www.andhrajyothy.com"],
    heuristicTransformFonts: ["Shree-Tel-0900"],

    //Preferences
    prefBranch:Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("Padma."),

    createTransformer : function() {
        this.transformer = Transformer.createTransformer(this.inputMethod, this.outputMethod);
        if (this.inputMethod == Transformer.method_RTS)
            this.transformer.setRTSMode(this.rtsStartMode);
        if (this.outputMethod == Transformer.method_RTS) {
            var RTSOutputFormat = this.prefBranch.prefHasUserValue("RTSOutputFormat") ? 
                                  this.prefBranch.getIntPref("RTSOutputFormat") : 0; //default = 0
            this.transformer.setRTSOutputFormat(RTSOutputFormat > 2 || RTSOutputFormat < 0 ? 0 : RTSOutputFormat);
        }
    },

    isHeuristicTransformEnabled: function() {
        return this.prefBranch.prefHasUserValue("enableHeuristics") ? this.prefBranch.getBoolPref("enableHeuristics") : true;
/*
        var whitelist = prefBranch.prefHasUserValue("heuristicWhiteList") ? prefBranch.getCharPref("heuristicWhiteList") : "";
        return whitelist.split(",");
*/
    },

    //From http://www.mozilla.org/docs/dom/technote/whitespace/nodomws.js
    is_all_ws: function(text) {
        return !(/[^\t\n\r ]/.test(text));
    },

    //multiple uri's should be separated by ,
    makeRegExOfURIS: function(list) {
        return new RegExp(list.replace(/www\./gi, '').replace(/,/gi, '|').replace(/\./gi, "\\."));
    },

    //Auto Transform
    onAutoTransformInit: function() {
        //Handle background loads in the browser
        var appcontent = document.getElementById("appcontent"); //browser
        if (appcontent)
            appcontent.addEventListener("DOMContentLoaded", this.onPageLoadPre, true);
        var messagepane = document.getElementById("messagepane");   //mail
        if (messagepane)
            messagepane.addEventListener("DOMContentLoaded", this.onPageLoadPre, true);
    },

    //The event target is the browser window, this is a hack to make Pamda_Browser_Transformer 'this'
    onPageLoadPre: function(evt) {
        Padma_Browser_Transformer.onPageLoad(evt);
    },

    //No heuristics - auto transform all text nodes that are children of this font node
    //TODO - if a font node is the child of another font node we will process it twice
    processFontNode: function(node, font) {
        for(var j = 0; j < node.childNodes.length; ++j) {
            var child = node.childNodes.item(j);
            if(child.nodeType == 3 && !this.is_all_ws(child.nodeValue)) {
                this.transformer.setDynamicFont(font);
                child.replaceData(0, child.length, this.transformer.convert(child.nodeValue));
            }
            else if (child.nodeType != 8)
                //No comment nodes
                this.processFontNode(child, font);
        }
    },

    //Apply heuristics to tree rooted at node
    //TOOD: remember a node's style while going down rather than going up again
    processTreeWithHeuristics: function(page, node, newfont) {
        for(var j = 0; j < node.childNodes.length; ++j) {
            var child = node.childNodes.item(j);
            if(child.nodeType == 3 && !this.is_all_ws(child.nodeValue)) {
                var parent = child.parentNode;
                while (parent.nodeType != 1 && parent.nodeType != 9)
                    parent = parent.parentNode;
        
                if (parent.nodeType == 1) {
                    var style = page.defaultView.getComputedStyle(parent, null);
                    var font = style.getPropertyValue("font-family");
                    if (this.transformer.setDynamicFont(font) == true || (
                            font == "serif" && this.transformer.setDynamicFont(newfont) == true))
                        child.replaceData(0, child.length, this.transformer.convert(child.nodeValue));
                }
            }
            else if (child.nodeType != 8)
                //Ignore comment nodes
                this.processTreeWithHeuristics(page, child, newfont);
        }
    },

    getAutoTransformSiteList: function() {
        var whitelist = "";
        if (this.prefBranch.prefHasUserValue("autoTransformWhiteList"))
            whitelist = this.prefBranch.getCharPref("autoTransformWhiteList");
        return this.makeRegExOfURIS(whitelist);
    },

    isAutoTransformEnabled: function() {
        return this.prefBranch.prefHasUserValue("enableAutoTransform") ? this.prefBranch.getBoolPref("enableAutoTransform") : true;
    },

    onPageLoad: function(evt) {
        var page = evt.originalTarget;
        if (!page || !page.location || page.location == "about:blank" || !page.location.host || !this.isAutoTransformEnabled())
            return;

        //Quickly check if auto transform is enabled and if this page needs to be transformed
        var whitelist = this.getAutoTransformSiteList();
        if (!whitelist.test(page.location.host))
            return;

        initializeRelationships();
        this.inputMethod = Transformer.method_DynFonts;
        this.outputMethod = Transformer.method_Unicode;
        this.createTransformer();

        //Do we need to apply heuristics
        if (this.isHeuristicTransformEnabled()) {
            for(var i = 0; i < this.heuristicTransformSites.length; ++i) {
                var regex = this.makeRegExOfURIS(this.heuristicTransformSites[i]);
                if (regex.test(page.location.host)) {
                    var body = page.getElementsByTagName("BODY");
                    for(var i = 0; i < body.length; ++i)
                        this.processTreeWithHeuristics(page, body[i], this.heuristicTransformFonts[i]);
                    return;
                }
            }
        }

        var elems = page.getElementsByTagName("FONT");
        for(var i = 0; i < elems.length; ++i) {
            var style = page.defaultView.getComputedStyle(elems[i], null);
            var font = style.getPropertyValue("font-family");
            if (this.transformer.setDynamicFont(font) == true)
                this.processFontNode(elems[i], font);
        }
    },

    //Manual transform starts here
    popupNodeBaseURI: null,

    //Based on input method, determine if the selected text needs to be transformed
    requiresTransform: function(node) {
        if (this.inputMethod == Transformer.method_Unknown) {
            if (node.nodeValue.match(/[\u0C00-\u0C7F]/))
                return true;
        }

        if (this.inputMethod == Transformer.method_DynFonts || this.inputMethod == Transformer.method_Unknown) {
            //Check computed style for parent node
            var parent = node.parentNode;
            while (parent.nodeType != 1 && parent.nodeType != 9)
                parent = parent.parentNode;
        
            if (parent.nodeType == 1) {
                var style = document.defaultView.getComputedStyle(parent, null);
                var font = style.getPropertyValue("font-family");
                if (this.transformer.setDynamicFont(font) == true) {
                    //if transforming to RTS, set font-family to Arial so that the right text is displayed even
                    //if the dynamic font is installed on the user machine
                    if (this.outputMethod == Transformer.method_RTS)
                        parent.style.fontFamily = "Arial";
                    return true;
                }

                if (font == "serif" && this.isHeuristicTransformEnabled()) {
                    for(var i = 0; i < this.heuristicTransformSites.length; ++i) {
                        var regex = this.makeRegExOfURIS(this.heuristicTransformSites[i]);
                        if (regex.test(this.popupNodeBaseURI))
                            return this.transformer.setDynamicFont(this.heuristicTransformFonts[i]);
                    }
                }
                return false;
            }
            return false;
        }
        return true;
    },

    //Process all nodes with root "root"
    processNodes: function(selection, root, start, end, soff, eoff) {
        if (root.nodeType == 3) {
            if (selection.containsNode(root, true) && this.requiresTransform(root)) {
                if (root == start && root == end)
                    root.replaceData(soff, eoff - soff, this.transformer.convert(root.nodeValue.substring(soff, eoff)));
                else if (root == start)
                    root.replaceData(soff, root.length - soff, this.transformer.convert(root.nodeValue.substring(soff)));
                else if (root == end)
                    root.replaceData(0, eoff, this.transformer.convert(root.nodeValue.substring(0, eoff)));
                else root.replaceData(0, root.length, this.transformer.convert(root.nodeValue));
            }
        }
        else if (root.nodeType != 8) {
            //Don't process comment nodes
            for(var i = 0; i < root.childNodes.length; ++i)
                this.processNodes(selection, root.childNodes.item(i), start, end, soff, eoff);
        }
    },

    traverseDOM: function(selection) {
        for(var i = 0; i < selection.rangeCount; i++) {
            var range = selection.getRangeAt(i);
            this.processNodes(selection, range.commonAncestorContainer, range.startContainer, range.endContainer, 
                              range.startOffset, range.endOffset);
        }
    },
    
    //Used only for special controls
    setTransformedText: function(text) {
        //The credit for this piece goes to Jed Brown of the BBCode extension.
        //http://jedbrown.net/1.0
        try {
            var command = "cmd_insertText";
            var controller = document.commandDispatcher.getControllerForCommand(command);
            
            if (controller && controller.isCommandEnabled(command)) {
                controller = controller.QueryInterface(Components.interfaces.nsICommandController);
                var params = Components.classes["@mozilla.org/embedcomp/command-params;1"];
                params = params.createInstance(Components.interfaces.nsICommandParams);
                
                params.setStringValue("state_data", text);
                controller.doCommandWithParams(command, params);
            }
        } 
        catch (e) {
            dump("Can't do cmd_insertText! ");
            dump(e+"\n");
        }
    },

    //Callback
    onManualTransform: function(inputMode, outputMode, rtsStart) {
        initializeRelationships();
        
        //Create a transformer
        this.inputMethod = inputMode;
        this.outputMethod = outputMode;
        this.rtsStartMode = rtsStart;
        this.createTransformer();
        
        var node = document.popupNode;
        var nodeLocalName = node.localName.toUpperCase();

        this.popupNodeBaseURI = node.baseURI;

        var selection;
        //Check for Selection in various controls
        //This code is thanks to Jaap Haitsma of the dictionarySearch extension and roachfiend of a million extensions
        //http://roachfiend.com/
        //http://mozdev.org/pipermail/project_owners/2004-March/001833.html
        if ((nodeLocalName == "TEXTAREA") || (nodeLocalName == "INPUT" && (node.type == "text" || node.type == "password"))) {
            selection = node.value.substring(node.selectionStart, node.selectionEnd);
        } 
        else if (nodeLocalName == "OPTION") {
            var parentSelect = node.parentNode;
            if (parentSelect.localName.toUpperCase() == "SELECT") {
                if (parentSelect.multiple) {
                    var anOption;
                    for (var i = 0; i < parentSelect.options.length; i++) {
                        anOption = parentSelect.options[i];
                        if (anOption.selected) {
                            selection += " " + anOption.value;
                        }
                    }
                } 
                else {
                    selection = node.value;
                }
            }
        } 
        else if (nodeLocalName == "SELECT") {
            selection = node.options[node.selected].value;
        }
        else {
            //Selection in a regular window
            var focusedWindow = document.commandDispatcher.focusedWindow;
            var selection = focusedWindow.getSelection();
            this.traverseDOM(selection);
            if (selection.rangeCount != 0)
                selection.collapseToEnd();
            return;
        }
        
        //For the specials
        this.setTransformedText(this.transformer.convert(selection));
    },



};

window.addEventListener("load", function() { Padma_Browser_Transformer.onAutoTransformInit(); }, false);
