// $Id: DynamicFontParser.js,v 1.1.2.5 2005/07/17 21:22:02 vnagarjuna Exp $ -->

//Copyright 2005 Nagarjuna Venna <vnagarjuna@yahoo.com>

/* ****** BEGIN LICENSE BLOCK ***** 
 *  This file is part of Padma.
 *
 *  Padma is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Padma is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Padma; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * ***** END LICENSE BLOCK ***** */

//Parser for text encoded using dynamic fonts
//1. Preprocess message - for most fonts this is just removing all unnecessary symbols
//2. Put prefix symbols in the right place 
//3. Break up combo symbols if necessary
//Get this right first, make it fast later

function DynamicFontParser(input, encoding) {
    function removeRedundantSymbols(input, encoding)
    {
        var output = "";
        //1.
        for(var i = 0; i < input.length; ++i)
            if (!encoding.isRedundant(input[i]))
                output += input[i];
        return output;
    }

    this.index = 0;
    //An encoding should implement either isRedundant or preprocessMessage
    if (encoding.preprocessMessage != undefined)
        this.text = encoding.preprocessMessage(input);
    else this.text = removeRedundantSymbols(input, encoding);
    this.length = this.text.length;
    this.encoding = encoding;
}

function DynamicFontParser_next()
{
    var response = "";
    this.prefix = "";
    for(var remaining = this.length - this.index; remaining > 0; remaining = this.length - this.index) {
        var maxLookupLen = remaining > this.encoding.maxLookupLen ? this.encoding.maxLookupLen : remaining;
            
        var key = null, value = null;
        for(var i = 1; i <= maxLookupLen; i++) {
            var lookup_key = this.text.substring(this.index, this.index + i);
            //alert("looking up key " + lookup_key);
            var lookup_value = this.encoding.lookup(lookup_key);
            if (lookup_value == null) {
                //alert("No match");
                if (key == null)
                    key = lookup_key;
                continue;
            }
            key = lookup_key;
            value = lookup_value;
            //alert("match for key " + key + " , match = " + value);
            if (!this.encoding.isOverloaded(lookup_key))
                break;
            //alert("key " + lookup_key + " is overloaded");
        }

        var output = this.handleInput(value, key, response, this.encoding.isPrefixSymbol(key));

        if (this.state == Parser.state_PREFIX)
            this.prefix += output;
        else if (this.state == Parser.state_START)
            return output;
        else response = output;
    }

    //always return what we have even if it does not make sense
    if (response.length != 0 || this.prefix.length != 0)
        return this.handleConsonantTermination(response);
    return null;
}

function DynamicFontParser_handleConsonantTermination(current)
{
    //3.
    var terminators = "", consnts = "";
    current += this.prefix;
    for(var i = 0; i < current.length; ++i) {
        var type = Padma.getType(current[i]);
        if (type == Padma.type_gunimtam || type == Padma.type_spaccu)
            terminators += current[i];
        else consnts += current[i];
    }

    if (terminators.length == 2) {
        var term1 = terminators[0], term2 = terminators[1];
        if ((term1 == Padma.vowelsn_E && term2 == Padma.vowelsn_AILEN) ||
            (term1 == Padma.vowelsn_AILEN && term2 == Padma.vowelsn_E))
            terminators = Padma.vowelsn_AI;
        else if ((term1 == Padma.vowelsn_E && term2 == Padma.vowelsn_EELEN) ||
            (term1 == Padma.vowelsn_EELEN && term2 == Padma.vowelsn_E))
            terminators = Padma.vowelsn_EE;
        else if ((term1 == Padma.vowelsn_E && term2 == Padma.vowelsn_AA) ||
            (term1 == Padma.vowelsn_AA && term2 == Padma.vowelsn_E))
            terminators = Padma.vowelsn_OO;
        else if ((term1 == Padma.vowelsn_E && term2 == Padma.vowelsn_U) ||
            (term1 == Padma.vowelsn_U && term2 == Padma.vowelsn_E))
            terminators = Padma.vowelsn_O;
    }
    return consnts + terminators;
}

//For dynamic fonts, we can recognize that the current syllable is complete only when the next one starts.
function DynamicFontParser_isCurrentSyllableComplete(type)
{
    return false;
}
