// $Id: EenaduParser.js,v 1.1.2.2 2005/07/10 20:46:45 vnagarjuna Exp $ -->

//Copyright 2005 Nagarjuna Venna <vnagarjuna@yahoo.com>

/* ****** BEGIN LICENSE BLOCK ***** 
 *  This file is part of Padma.
 *
 *  Padma is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Padma is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Padma; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * ***** END LICENSE BLOCK ***** */

//Parser for Eenadu Text - returns next syllable
//1. Remove all unnecessary symbols (talakattu, 0x2d)
//2..Put prefix symbols in the right place 
//3. Break up combo symbols if necessary
//Get this right first, make it fast later

function EenaduParser(input) {
    function removeRedundantSymbols(input)
    {
        var output = "";
        //1.
        for(var i = 0; i < input.length; ++i) {
            var str = input.substring(i, i + 1);
            if (!Eenadu.isRedundant(str))
                output += str;
        }
        return output;
    }

    this.index = 0;
    this.text = removeRedundantSymbols(input);
    this.length = this.text.length;
}

function EenaduParser_next()
{
    var syllable = "", prefix = "", stem = false, ctxt = Eenadu.ctxt_REGULAR;
    for(var remaining = this.length - this.index; remaining > 0; remaining = this.length - this.index) {
        var maxLookupLen = remaining > Eenadu.maxLookupLen ? Eenadu.maxLookupLen : remaining;
            
        var key = null, value = null;
        for(var i = maxLookupLen; i > 0; i--) {
            key = this.text.substring(this.index, this.index + i);
            //alert("looking up key " + key);
            value = Eenadu.lookup(key, ctxt);
            if (value != null) {
                //alert("match for key " + key + " , match = " + value);
                break;
            }
            else {
                //alert("No match");
            }
        }

        //No more match, return already matched text if available otherwise the key
        if (value == null) {
            if (syllable.length == 0 && prefix.length == 0) {
                //alert("returning key " + key);
                this.index += key.length;
                return key;
            }
            break;
        }

        //2.
        if (syllable.length == 0) {
            if (Eenadu.isPrefixSymbol(key))
                prefix += value;
            else {
                syllable += value;
                ctxt = Eenadu.getContext(key);
            }
        }
        else if (syllable.length != 0 && Eenadu.isSyllableStart(key)) {
            break;
        }
        else {
            syllable += value;
        }
        this.index += key.length;
    }

    //3.
    if (syllable.length != 0 || prefix.length != 0) {
        var output = "", specials = "";
        
        syllable += prefix;
        for(var i = 0; i < syllable.length; ++i) {
            var str = syllable.substring(i, i + 1);
            if (Eenadu.isSpecialValue(str))
                specials += str;
            else output += str;
        }
        //Hack
        if (specials.length == 2) {
            var str1 = specials.substring(0, 1);
            var str2 = specials.substring(1, 2);
            if ((str1 == Padma.vowelsn_E && str2 == Padma.vowelsn_AILEN) || 
                (str1 == Padma.vowelsn_AILEN && str2 == Padma.vowelsn_E))
                specials = Padma.vowelsn_AI;
        }
        output += specials;
        //alert("returning " + output);
        return output;
    }
    return null;
}
