// $Id: RTSParser.js,v 1.1.2.4 2005/07/10 20:46:45 vnagarjuna Exp $ -->

//Copyright 2005 Nagarjuna Venna <vnagarjuna@yahoo.com>

/* ****** BEGIN LICENSE BLOCK ***** 
 *  This file is part of Padma.
 *
 *  Padma is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.

 *  Padma is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Padma; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * ***** END LICENSE BLOCK ***** */

//Parser for RTS Text - returns syllables

function RTSParser(input) {
    this.index = 0;
    this.length = input.length;
    this.text = input;
}

function RTSParser_updateSymbolType(type)
{
    if (this.state != Parser.state_CONSNT)
        return type;
    switch (type) {
        case Padma.type_accu:
            return Padma.type_gunimtam;

        case Padma.type_hallu:
            return Padma.type_vattu;

        default:
            return type;
    }
}

function RTSParser_next()
{
    var response = "";

    var remaining = this.length - this.index;
    for(; remaining > 0; remaining = this.length - this.index) {
        var maxLookupLen = remaining > RTS.maxLookupLen ? RTS.maxLookupLen : remaining;
        
        var value, key;
        for(var i = maxLookupLen; i > 0; i--) {
            key = this.text.substring(this.index, this.index + i);
            value = RTS.lookup(key);
            if (value != null) {
                //found a match
                if (!RTS.isAnusvara(key)) {
                    break;
                }
                //the match is one of the special sunnas (n or m followed by some consonant). Treat this as sunna
                //only if this is the beginning of a syllable. Lame example: #arnku# - the nk here should not be sunna
                //This piece of code is probably not necessary, I doubt there is a word in telugu that would match this
                else if (response.length == 0) {
                    //only the n/m is consumed - so key is 1 less
                    key = this.text.substring(this.index, this.index + i - 1);
                    break;
                }
            }
        }

        response = this.handleInput(value, key, response);
        if (this.state == Parser.state_START)
            return response;
    }

    if (response.length != 0) {
        return this.handleConsonantTermination(response);
    }
    return null;
}

function RTSParser_handleConsonantTermination(current)
{
    if (current == Padma.consnt_MA) {
        //alert("3: returning Anusvara");
        return Padma.anusvara;
    }
    //alert("4: returning " + current + Padma.pollu);
    return current + Padma.pollu;
}

